#include <fstream>
#include <iostream>
#include <utility>
#include <map>
#include <cstdlib>
#include <string>

enum
{
  OUTCOME_OK,
  OUTCOME_WA,
  OUTCOME_PE,
  OUTCOME_FAIL,
};

enum
{
  PIPE_HORIZONTAL = '',
  PIPE_VERTICAL = '',
  PIPE_CORNER_UL = '',
  PIPE_CORNER_UR = '',
  PIPE_CORNER_DL = '',
  PIPE_CORNER_DR = '',
  PIPE_CROSS = '',
  PIPE_NONE = ' ',
  PIPE_CROSS_BROKEN = 'B',
  PIPE_WASTED = 'W',
};

const int maxMatSize = 25;
const int iShift[] = {0, 1, 0, -1};
const int jShift[] = {1, 0, -1, 0};

int crossHorizontalDirection[maxMatSize][maxMatSize];
int crossVerticalDirection[maxMatSize][maxMatSize];
int mat[maxMatSize][maxMatSize];

inline int isSymCorner(char c)
{
  return c == PIPE_CORNER_UL || c == PIPE_CORNER_UR || c == PIPE_CORNER_DL || c == PIPE_CORNER_DR;
}

void die(const char* message, int outcome)
{
  std::cout << message;
  exit(outcome);
}

int main (int argc, char** argv)
{
  std::string inputFilename = argv[1];
  std::string outputFilename = argv[2];
  std::string answerFilename = argv[3];
  
  std::fstream input(inputFilename.c_str(), std::ios_base::in);

  int startI = -1, startJ = -1, crossesInInput = 0, segmentsInInput = 0;
  int w, h;
  input >> h >> w;
  std::string foo;
  std::getline(input, foo);
 
  for (int i = 0; i < h; i++)
  {
    std::string line;
    std::getline(input, line);

    for (int j = 0; j < w; ++j)
    {
      mat[i][j] = line[j];

      if (mat[i][j] != PIPE_HORIZONTAL &&
          mat[i][j] != PIPE_VERTICAL &&
          mat[i][j] != PIPE_CORNER_UL &&
          mat[i][j] != PIPE_CORNER_UR &&
          mat[i][j] != PIPE_CORNER_DL &&
          mat[i][j] != PIPE_CORNER_DR &&
          mat[i][j] != PIPE_CROSS &&
          mat[i][j] != PIPE_NONE) die ("Test file contains invalid characters", OUTCOME_FAIL);

      if (mat[i][j] != PIPE_NONE)
      {
        if (startI == -1 && startJ == -1)
        {
          startI = i;
          startJ = j;
        }
        segmentsInInput++;
      }
      if (mat[i][j] == PIPE_CROSS) crossesInInput++;
    }
  }

  std::fstream answer(answerFilename.c_str(), std::ios_base::in);
  int juryBrokenCrossings;
  answer >> juryBrokenCrossings;

  std::fstream output(outputFilename.c_str(), std::ios_base::in);
  int participantBrokenCrossings;
  output >> participantBrokenCrossings;
  for (int curCrossing = 0; curCrossing < participantBrokenCrossings; ++curCrossing)
  {
    int r, c;
    output >> r >> c;
    if (output.fail()) die("Unable to read participant program output!", OUTCOME_PE);

    if (mat[r][c] == PIPE_CROSS_BROKEN) die("Some crossing is broken more than once!", OUTCOME_WA);
    if (mat[r][c] != PIPE_CROSS) die("Specified coordinates do not hit any crossing!", OUTCOME_WA);
    mat[r][c] = PIPE_CROSS_BROKEN;       
  }

//  if (!output.eof()) die("Extra input!", OUTCOME_PE);

  if (juryBrokenCrossings > participantBrokenCrossings) die("Jury has better solution, with more broken crossings!", OUTCOME_WA);

  std::map<std::pair<int, int>, char > clockwiseTurnSymbol;
  clockwiseTurnSymbol[std::make_pair(0, 1)] = PIPE_CORNER_DL;
  clockwiseTurnSymbol[std::make_pair(1, 2)] = PIPE_CORNER_UL;
  clockwiseTurnSymbol[std::make_pair(2, 3)] = PIPE_CORNER_UR;
  clockwiseTurnSymbol[std::make_pair(3, 0)] = PIPE_CORNER_DR;

  int i = startI;
  int j = startJ;
  int dir = 0;
  int visited = 0;
  while (visited < segmentsInInput + crossesInInput)
  {
    visited++;
    i += iShift[dir];
    j += jShift[dir];

    if (isSymCorner(mat[i][j]))
    {
      if (mat[i][j] != clockwiseTurnSymbol[std::make_pair(dir, (dir + 1) % 4)]) die ("Wrong turn direction in input file!", OUTCOME_FAIL);
      dir = (dir + 1) % 4;
    }
    else
    if (mat[i][j] == PIPE_VERTICAL || mat[i][j] == PIPE_HORIZONTAL)
    {
      if (mat[i][j] == PIPE_VERTICAL && dir % 2 == 0) die ("Wrong pipe direction in input file!", OUTCOME_FAIL);
      if (mat[i][j] == PIPE_HORIZONTAL && dir % 2 != 0) die ("Wrong pipe direction in input file!", OUTCOME_FAIL);
    }
    else
    if (mat[i][j] == PIPE_CROSS_BROKEN || mat[i][j] == PIPE_CROSS)
    {
      if (dir % 2 == 0) crossHorizontalDirection[i][j] = 2 * (dir == 0) - 1;
      if (dir % 2 == 1) crossVerticalDirection[i][j] = 2 * (dir == 1) - 1;
    }
  }

  i = startI;
  j = startJ;
  dir = 0;
  int visitedAfterCrossesBreakage = 0;
  while (mat[i][j] != PIPE_WASTED)
  {
    visitedAfterCrossesBreakage++;
    if (mat[i][j] != PIPE_CROSS && mat[i][j] != PIPE_CROSS_BROKEN) mat[i][j] = PIPE_WASTED;
    i += iShift[dir];
    j += jShift[dir];

    if (isSymCorner(mat[i][j]))
    {
      dir = (dir + 1) % 4;
    }
    else
    if (mat[i][j] == PIPE_CROSS_BROKEN)
    {
      if (dir % 2 == 0)
        dir = (crossVerticalDirection[i][j] == 1) ? 1 : 3; 
      else
        dir = (crossHorizontalDirection[i][j] == 1) ? 0 : 2;
    }
  }

  if (visitedAfterCrossesBreakage < visited) die("Connectedness violated!", OUTCOME_WA);

  die("Seems good!", OUTCOME_OK);

  return 0;
}
